@@ -353,7 +353,9 @@ class HTTPResponse(httplib.HTTPResponse):
 
 
     def __init__(self, sock, debuglevel=0, strict=0, method=None):
-        httplib.HTTPResponse.__init__(self, sock, debuglevel, method)
+        httplib.HTTPResponse.__init__(self, sock, debuglevel=debuglevel,
+                                      strict=True, method=method,
+                                      buffering=True)
         self.fileno = sock.fileno
         self.code = None
         self._rbuf = ''
